//--------------------------------------------------------------------------------------------------------------------------//
// Setting up WLAN and a Webserver
//--------------------------------------------------------------------------------------------------------------------------//

// There is multiple ways of doing this, so this is just one option.
// This is how to set up an Asyncronous Websever, for displaying, sending, etc

// Quick note: The WiFi functions can uptake certain pins, making them unavailable for other manual use

#include <Arduino.h>
#include <WiFi.h> // Neccessary whenever you're planning on using WiFi

// Include Webserver Library if planning on using //
#include <ESPAsyncWebServer.h> // Not found?
                               // No problem. To install it, follow the instructions in the .docx, just replace the name with "esp ayncwebsever"
                               // to get the right results. 
                               // Select the correct one, Author is "ESPHome Team"
                               // There may be some issues with the library at one point, so
                               // if that's the case, do it the manual way.
                               // That means throwing everything I put in a folder called "Lib_backup", cut out it's contents and add them
                               // to your source folder.
                               // That method for sure is foolproof.

#include <SPIFFS.h> // IF you are planning on hosting an html page with or without css, this is the correct way of doing it.
#include <SPIFFSEditor.h> // Necessary to go along with this, copy "SPIFFSEditor.cpp" AND "SPIFFSEditor.h" from the src folder if needed. (You'll notice when using !SPIFFS)

// Set necessary variables, can also be phrased as "#define ..." //
const char* _ssid = "Here the name of your WiFi"; // For example: "cpsLabor2.4"
const char* _password = "password to that WiFi here";

// Create an instance of the Webserver library / class //
AsyncWebServer server(80);

// Now, if you want to send and update the page and need values from your running programm, you'll need a "processor" function.
String processor(const String& _var) 
{
    Serial.println(_var);
    if(_var == "STATE") // == Keyword from html
    {
        return ""; // Value to be handed over to the html file for displaying
    }
    return String();
}
// It's important to note that the value being handed over is in the String format, so be careful not to hand over anything else.
// (The reason for that? HTML displays all the text as Strings, and therefor does not expect integer or floats for example.)

void setup()
{
    // Set up Spiffs, initializing it //
    if(!SPIFFS.begin(true))
    {
        Serial.println("An Error has occured while mounting SPIFFS.");
        return;
    }

    WiFi.begin(_ssid, _password); // Start WLAN connection.
    while(WiFi.status() != WL_CONNECTED) // Waiting for stable wifi connection.
    {
        delay(1000);
        Serial.println("Connecting to Wifi...");
    }

    Serial.println(WiFi.localIP()); // Print the IP assigned to the ESP.

    server.on("/", HTTP_GET, [](AsyncWebServerRequest *request) // Upon call of the page IP (accessed through the ESP), receive a request. You can change this to auto update, look up if interested.
    {
        request->send(SPIFFS, "/index.html", String(), false, processor); // What should be called upon request? Index.html == Mainpage / html file.
    });
    server.on("/style.css", HTTP_GET, [](AsyncWebServerRequest *request) // If the server wants a styling / supports one
    {
    request->send(SPIFFS, "/style.css", "text/css"); // Respond with the right css file, main for now.
    });

    // Start the server //
    server.begin();
}

void loop()
{

}

// Now... how do you use SPIFFS? //
// It's a bit harder to understand, but Spiffs is a filesystem that is compatible with microkontrollers like the ESP32.
// WATCH OUT!! It is NOT compatible with ESP S2, but ESP32 S1 can handle it more than well.
// You might think... why would I need that?
// If you want to host a Webserver and display a website, you can either write the HTML in the C code directly, which is not only tiresome,
// but also ineffecient (and it looks like throw up imo), or you store the html and css files, as well as any external sources such as pictures, in a new
// and organized folder called "data". And to work with that folder and it's contents, you need SPIFFS.

// First off all, create a new folder in your project and call it "data" like I mentioned. Leave it empty for now.
// In this project, there is already a "data" folder, so here it isn't needed anymore. In there, you store your html and css files, and whatever
// else you could need.
// Once done setting up Spiffs, go check out the contents of said folder!


// But how do I set it up? //
// You first of all rearrange the partitions of your ESP in your project folder. For that, you need something called a ".csv" file,
// that helps you to divide your flash memory (You need to store your files there and nowhere else or they're gone) and helps you assign
// those partitions to certain uses. Such as OTA (over the air), or for example app assigned memory, coredump, etc etc

// For that, check out the .csv file in this project and take it as a template. Read further there for more information.